﻿using System;
using System.Drawing;
using Telerik.WinControls.Enumerations;
using Telerik.WinControls.UI;

namespace CheckBoxInHeader
{
    public class CheckBoxHeaderCell : GridHeaderCellElement
    {
        #region Fields

        private RadCheckBoxElement checkbox;
        private int suspendNotificationCount = 0;

        #endregion

        #region Initialization

        public CheckBoxHeaderCell(GridViewColumn column, GridRowElement row)
            : base(column, row)
        {

        }

        protected override void InitializeFields()
        {
            base.InitializeFields();
            this.DrawText = false;
        }

        public override void Initialize(GridViewColumn column, GridRowElement row)
        {
            base.Initialize(column, row);
            column.AllowSort = false;
        }

        protected override void DisposeManagedResources()
        {
            this.checkbox.ToggleStateChanged -= new StateChangedEventHandler(checkbox_ToggleStateChanged);
            base.DisposeManagedResources();
        }

        protected override void CreateChildElements()
        {
            base.CreateChildElements();
            this.checkbox = new RadCheckBoxElement();
            this.checkbox.ToggleStateChanged += new StateChangedEventHandler(checkbox_ToggleStateChanged);
            this.Children.Add(checkbox);
        }

        #endregion

        #region Properties

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(GridHeaderCellElement);
            }
        }

        public override object Value
        {
            get
            {
                return Convert.ToBoolean(this.RowInfo.Tag);
            }
            set
            {
                this.RowInfo.Tag = value;
            }
        }

        #endregion

        #region Methods

        protected override void SetContentCore(object value)
        {
            base.SetContentCore(value);

            this.suspendNotificationCount++;

            if (this.RowInfo.Tag != null)
            {
                this.checkbox.ToggleState = (bool)value ? ToggleState.On : ToggleState.Off;
            }
            else
            {
                this.checkbox.ToggleState = ToggleState.Off;
            }

            this.checkbox.TextElement.Text = this.Text;

            this.suspendNotificationCount--;
        }

        public override bool IsCompatible(GridViewColumn data, object context)
        {
            return data.Name == "Select" && context is GridTableHeaderRowElement
                && base.IsCompatible(data, context);
        }


        #endregion

        #region Event Handlers

        private void checkbox_ToggleStateChanged(object sender, StateChangedEventArgs args)
        {
            if (this.suspendNotificationCount > 0)
            {
                return;
            }

            bool value = args.ToggleState == ToggleState.On;
            this.RowInfo.Tag = value;
            this.GridViewElement.EditorManager.EndEdit();
            this.TableElement.BeginUpdate();

            GridTraverser traverser = new GridTraverser(this.ViewInfo, GridTraverser.TraversalModes.AllRows);

            while (traverser.MoveNext())
            {
                GridViewRowInfo rowInfo = traverser.Current;

                if (rowInfo is GridViewGroupRowInfo)
                {
                    rowInfo.Tag = value;
                }
                else
                {
                    rowInfo.Cells[this.ColumnIndex].Value = value;
                }
            }

            this.TableElement.EndUpdate(false);

            this.TableElement.Update(GridUINotifyAction.DataChanged);
        }

        #endregion

        #region Layout

        protected override SizeF ArrangeOverride(SizeF finalSize)
        {
            SizeF size = base.ArrangeOverride(finalSize);
            RectangleF rect = GetClientRectangle(finalSize);
            this.checkbox.Arrange(new RectangleF((finalSize.Width - this.checkbox.DesiredSize.Width) / 2, (rect.Height - 20) / 2, 20, 20));

            return size;
        }

        #endregion
    }
}
