﻿using System;
using System.Drawing;
using Telerik.WinControls.Enumerations;
using Telerik.WinControls.UI;

namespace CheckedGroupHeaderCellElement
{
    public class CustomGroupHeaderCell : GridGroupContentCellElement
    {
        #region Fields

        private RadCheckBoxElement checkbox;
        private int suspendNotificationCount = 0;

        #endregion

        #region Initialization

        public CustomGroupHeaderCell(GridViewColumn column, GridRowElement row)
            : base(column, row)
        {

        }

        protected override void InitializeFields()
        {
            base.InitializeFields();
            this.DrawText = false;
        }

        protected override void CreateChildElements()
        {
            base.CreateChildElements();

            this.checkbox = new RadCheckBoxElement();
            this.checkbox.MinSize = new System.Drawing.Size(5, 5);
            this.checkbox.ToggleStateChanged += new StateChangedEventHandler(checkbox_ToggleStateChanged);
            this.Children.Insert(0, checkbox);
        }

        protected override void DisposeManagedResources()
        {
            this.checkbox.ToggleStateChanged -= new StateChangedEventHandler(checkbox_ToggleStateChanged);
            base.DisposeManagedResources();
        }

        #endregion

        #region Properties

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(GridGroupContentCellElement);
            }
        }

        public override object Value
        {
            get
            {
                return Convert.ToBoolean(this.RowInfo.Tag);
            }
            set
            {
                this.RowInfo.Tag = value;
            }
        }

        #endregion

        #region Methods

        protected override void SetContentCore(object value)
        {
            base.SetContentCore(value);

            this.suspendNotificationCount++;

            if (this.RowInfo.Tag != null)
            {
                this.checkbox.ToggleState = (bool)value ? ToggleState.On : ToggleState.Off;
            }
            else
            {
                this.checkbox.ToggleState = ToggleState.Off;
            }

            this.checkbox.TextElement.Text = this.Text;

            this.suspendNotificationCount--;
        }

        #endregion

        #region Event Handlers

        private void checkbox_ToggleStateChanged(object sender, StateChangedEventArgs args)
        {
            if (this.suspendNotificationCount > 0)
            {
                return;
            }

            bool value = args.ToggleState == ToggleState.On;
            this.RowInfo.Tag = value;
            this.GridViewElement.EditorManager.EndEdit();
            this.TableElement.BeginUpdate();

            for (int i = 0; i < this.RowInfo.ChildRows.Count; i++)
            {
                this.RowInfo.ChildRows[i].Cells["FirstSelect"].Value = value;
            }

            IHierarchicalRow parent = this.RowInfo.Parent;

            if (value)
            {
                foreach (GridViewRowInfo row in parent.ChildRows)
                {
                    if (!Convert.ToBoolean(row.Tag))
                    {
                        value = false;
                        break;
                    }
                }
            }

            GridViewRowInfo rowInfo = parent as GridViewRowInfo;
            GridViewTemplate template = parent as GridViewTemplate;

            if (rowInfo != null)
            {
                template = rowInfo.ViewTemplate;
            }

            if (template != null)
            {
                template.MasterViewInfo.TableHeaderRow.Tag = value;
            }

            this.TableElement.EndUpdate(false);
            this.TableElement.Update(GridUINotifyAction.DataChanged);
        }

        #endregion

        #region Layout

        protected override SizeF ArrangeOverride(SizeF finalSize)
        {
            SizeF size = base.ArrangeOverride(finalSize);
            RectangleF rect = this.GetClientRectangle(finalSize);

            if (this.checkbox != null)
            {
                this.checkbox.Arrange(new RectangleF(rect.Width / 2 - this.checkbox.DesiredSize.Width / 2,
                                                     rect.Top + (rect.Height - this.checkbox.DesiredSize.Height) / 2,
                                                     this.checkbox.DesiredSize.Width,
                                                     this.checkbox.DesiredSize.Height));
            }

            return size;
        }

        #endregion

    }
}
